using System;
using System.ComponentModel;
using System.Web.UI;
using System.Web.UI.HtmlControls;

namespace captcha
{
	/// <summary>
	/// CAPTCHA prvok, ktory je urceny na rozlisenie cloveka od pocitaca.
	/// </summary>
	[DefaultProperty("TagKey"),
	 ToolboxData("<{0}:CaptchaControl runat=\"server\" />"),
	 ControlBuilder(typeof(captcha.Designer.EmptyControlBuilder))]
	public class CaptchaControl : System.Web.UI.Control, IValidator, INamingContainer
	{

		private HtmlInputText inputText;

		// ------------------------------------------------------------------
		/// <summary>
		/// OnInit
		/// </summary>
		/// <param name="e"></param>
		protected override void OnInit(EventArgs e)
		{
			base.OnInit (e);

			// hned po inicializovani sa pridame do stranky medzi validatory - aby
			// nas stranka pri validovani 'volala'
			this.Page.Validators.Add(this);

			this.EnsureChildControls();
		}

		// ------------------------------------------------------------------
		/// <summary>
		/// Create child controls
		/// </summary>
		protected override void CreateChildControls()
		{
			// Vsetky kompozitne "controly" by mali vytvarat svojich 'potomkov'
			// v metode CreateChildControls()

			// vytvorime control
			this.inputText = new System.Web.UI.HtmlControls.HtmlInputText("password");
			this.inputText.ID = "input";
			this.inputText.MaxLength = 3;
			// vlozime ho ako potomka
			this.Controls.Add( this.inputText );
		}


		// ------------------------------------------------------------------
		/// <summary>
		/// Sends server control content to a provided <see cref="HtmlTextWriter"/> object,
		/// which writes the content to be rendered on the client.
		/// </summary>
		/// <param name="writer"></param>
		protected override void Render(System.Web.UI.HtmlTextWriter writer)
		{
							
			// vyrenderujeme id-cko
			if (this.ID != null) 
			{
				writer.AddAttribute(HtmlTextWriterAttribute.Id, this.ClientID);
			}
			
			// vyrenderujeme CSS classu
			if (this.CssClass != null)  
			{
				writer.AddAttribute(HtmlTextWriterAttribute.Class, this.CssClass);
			}
					
			// zaciatocna znacka
			writer.RenderBeginTag(this.TagKey);

			writer.WriteBeginTag( "img" );
			writer.WriteAttribute("src", this.ResolveUrl(this.Url));
			writer.WriteAttribute("alt", String.Empty);
			writer.Write( HtmlTextWriter.TagRightChar );

			// zaciatok tela
			writer.AddAttribute(HtmlTextWriterAttribute.Class, "body");
			writer.RenderBeginTag("div");
			
			// obsah
			writer.AddAttribute(HtmlTextWriterAttribute.For, this.inputText.ClientID);
			writer.RenderBeginTag("label");
			writer.Write( "Zadaj kd ktor vid na obrzku:" );
			writer.RenderEndTag();

			// vyrenderujeme input formular
			this.RenderChildren( writer );
			
			// koniec tela
			writer.RenderEndTag();

			// koncova znacka
			writer.RenderEndTag();

		}

		#region IValidator Members

		/// <summary>
		/// <c>true</c> = tento control je viditelny;
		/// <c>false</c> = tento control nieje viditelny
		/// </summary>
		protected bool IsVisible
		{
			get 
			{
				Control control1;
				if (!base.Visible)
				{
					// ak je Validator skryty
					return false;
				}
				for (control1 = base.Parent; (control1 != null); control1 = control1.Parent)
				{
					// ak je Validator v skrytom 'otcovi' tak tiez false
					if (!control1.Visible)
					{
						return false;
					}
				}
				return true;
			}
		}

		// ------------------------------------------------------------------
		/// <summary>
		/// Skontroluje, ci je CAPTCHA kod spravny
		/// </summary>
		public void Validate()
		{
			if (!this.IsVisible)
			{
				this.isValid = true;
				return;
			}
			this.isValid = this.EvaluateIsValid();
		}

		// ------------------------------------------------------------------
		/// <summary>
		/// Prebieha samotna kontrola CAPTCHA kodu
		/// </summary>
		/// <returns></returns>
		protected virtual bool EvaluateIsValid()
		{
			// ak uzivatel nic nezadal tak konec
			if (this.inputText.Value.Length == 0) 
			{
				return false;
			}

			object o = Context.Session[ CaptchaHttpHandler.SESSION_CaptchaKey ];
			if ( !(o is string) ) 
			{
				return false;
			}

			string generated = (string)o;
			if (generated.Equals(this.inputText.Value))
			{
				return true;
			}
			return false;
		}

		private bool isValid = true;

		/// <summary>
		/// <c>true</c> - ak nieje ziadna chyba; inac <c>false</c>
		/// </summary>
		public bool IsValid
		{
			get
			{
				return this.isValid;
			}
			set
			{
				this.isValid = value;
			}
		}

		private string errorMessage;

		/// <summary>
		/// Chybova sprava, v pripade chyby <seealso cref="IsValid"/>
		/// </summary>
		public string ErrorMessage
		{
			get
			{
				
				return this.errorMessage;
			}
			set
			{
				this.errorMessage = value;
			}
		}

		#endregion
	
		private Random random = new Random();

		// ------------------------------------------------------------------
		/// <summary>
		/// Vygenerujeme nahodny kod
		/// </summary>
		/// <param name="length">dlzka kodu, ktory sa ma vygenerovat</param>
		/// <returns>nahodny kod zadanej dlzky</returns>
		private string GenerateRandomCode(int length)
		{
			string s = String.Empty;
			System.Text.StringBuilder sb = new System.Text.StringBuilder(length);
			for (int i = 0; i < length; i++) 
			{
				sb.Append( this.random.Next(10).ToString() );
			}
			return sb.ToString();
		}

		// ------------------------------------------------------------------
		/// <summary>
		/// On prerender
		/// </summary>
		/// <param name="e"></param>
		protected override void OnPreRender(EventArgs e)
		{
			base.OnPreRender (e);
			if (this.IsVisible) 
			{
				string captchaKey = this.GenerateRandomCode(3);
				this.Context.Session[ CaptchaHttpHandler.SESSION_CaptchaKey ] = captchaKey;
			}
		}

		// ------------------------------------------------------------------
		/// <summary>
		/// URL, odkial sa ma zobrazovat CAPTCHA obrazok; Za URL sa este doplni ako
		/// parameter nahodny kluc (ktory sluzi len koli tomu, aby browser tento
		/// obrazok necachoval)
		/// </summary>
		[Browsable(true), Category("Appearance"), DefaultValue("~/captchaimage.axd"), Description("URL, odkial sa ma zobrazovat CAPTCHA obrazok")]
		public string Url = "~/captchaimage.axd";

		// ------------------------------------------------------------------
		/// <summary>
		/// Base tag for this control. Default is 'div'
		/// </summary>
		[Browsable(true), Category("Appearance"), DefaultValue("div"), Description("Base tag for this control. Default is 'div'")]
		public string TagKey = "div";

		// ------------------------------------------------------------------
		/// <summary>
		/// Gets or sets the Cascading Style Sheet (CSS) class rendered by the
		/// Web server control on the client.
		/// </summary>
		[Browsable(true), Category("Appearance"), Description("Cascading Style Sheet (CSS) class rendered by the Web server control on the client")]
		public string CssClass = "captcha";

	} // public class
} // namespace
